"""Models used by the static media management application."""
from __future__ import unicode_literals

from PIL import Image

from django.db import models
from django.contrib import admin
from django.contrib.admin.widgets import ForeignKeyRawIdWidget
from django.utils.encoding import python_2_unicode_compatible

import os


@python_2_unicode_compatible
class Label(models.Model):

    """
    A notional label used to organise static media.

    This does not correspond to a physical label on the disk.
    """

    name = models.CharField(max_length=200)

    def __str__(self):
        """Returns the name of the label."""
        return self.name

    class Meta:
        ordering = ("name",)


@python_2_unicode_compatible
class File(models.Model):

    """A static file."""

    title = models.CharField(max_length=200,
                             help_text="The title will be used as the default rollover text when this media is embedded in a web page.")

    labels = models.ManyToManyField(
        Label,
        blank=True,
        help_text="Labels are used to help organise your media. They are not visible to users on your website.",
    )

    file = models.FileField(
        upload_to="uploads/files",
        max_length=250,
    )

    def get_absolute_url(self):
        """Generates the absolute URL of the image."""
        return self.file.url

    def __str__(self):
        """Returns the title of the media."""
        return self.title

    class Meta:
        ordering = ("title",)

    def is_image(self):
        from .admin import FILE_ICONS, IMAGE_FILE_ICON, UNKNOWN_FILE_ICON

        _, extension = os.path.splitext(self.file.name)
        extension = extension.lower()[1:]
        icon = FILE_ICONS.get(extension, UNKNOWN_FILE_ICON)
        return icon == IMAGE_FILE_ICON

    def width(self):
        if self.is_image():
            with open(self.file.path, "rb") as f:
                try:
                    image = Image.open(f)
                except IOError:
                    return 0
            image.verify()
            return image.size[0]
        return 0

    def height(self):
        if self.is_image():
            with open(self.file.path, "rb") as f:
                try:
                    image = Image.open(f)
                except IOError:
                    return 0
            image.verify()
            return image.size[1]
        return 0


class FileRefField(models.ForeignKey):

    """A foreign key to a File, constrained to only select image files."""

    def __init__(self, **kwargs):
        kwargs["to"] = 'media.File'
        kwargs.setdefault("related_name", "+")
        kwargs.setdefault("on_delete", models.PROTECT)
        super(FileRefField, self).__init__(**kwargs)

    def formfield(self, **kwargs):
        defaults = {
            "widget": ForeignKeyRawIdWidget(self.rel, admin.site),
        }
        return super(FileRefField, self).formfield(**defaults)


IMAGE_FILTER = {
    "file__iregex": r"\.(png|gif|jpg|jpeg)$"
}


class ImageRefField(FileRefField):

    """A foreign key to a File, constrained to only select image files."""

    def __init__(self, **kwargs):
        kwargs["limit_choices_to"] = IMAGE_FILTER
        super(ImageRefField, self).__init__(**kwargs)


VIDEO_FILTER = {
    "file__iregex": r"\.(webm|mp4|m4v)$"
}


class VideoFileRefField(FileRefField):

    """A foreign key to a File, constrained to only select video files."""

    def __init__(self, **kwargs):
        kwargs["limit_choices_to"] = VIDEO_FILTER
        super(VideoFileRefField, self).__init__(**kwargs)


@python_2_unicode_compatible
class Video(models.Model):

    title = models.CharField(
        max_length=200,
    )

    image = ImageRefField(
        blank=True,
        null=True,
    )

    high_resolution_mp4 = VideoFileRefField(
        verbose_name="high resolution MP4",
        blank=True,
        null=True,
    )

    low_resolution_mp4 = VideoFileRefField(
        verbose_name="low resolution MP4",
        blank=True,
        null=True
    )

    webm = VideoFileRefField(
        verbose_name="WebM",
        blank=True,
        null=True,
    )

    def __str__(self):
        """Returns the title of the media."""
        return self.title

    class Meta:
        ordering = ("title",)


class VideoRefField(models.ForeignKey):

    """A foreign key to a File, constrained to only select image files."""

    def __init__(self, **kwargs):
        kwargs["to"] = Video
        kwargs.setdefault("related_name", "+")
        kwargs.setdefault("on_delete", models.PROTECT)
        super(VideoRefField, self).__init__(**kwargs)

    def formfield(self, **kwargs):
        defaults = {
            "widget": ForeignKeyRawIdWidget(self.rel, admin.site),
        }
        return super(VideoRefField, self).formfield(**defaults)



from django.contrib import admin
from django.test import TestCase

from ..models import Article, Category

import sys
import mock


class TestArticleAdminBase(TestCase):

    def test_article_admin(self):
        del sys.modules['cms.apps.news.admin']

        admin.site.unregister(Article)
        admin.site.unregister(Category)

        with mock.patch('cms.apps.news.admin.externals.reversion', new=False):
            from ..admin import ArticleAdmin, ArticleAdminBase
            assert ArticleAdmin
            self.assertIs(ArticleAdmin.__bases__[0], ArticleAdminBase)
            self.assertEqual(len(ArticleAdmin.__bases__), 1)



"""Views used by the pages app."""

from django.contrib.contenttypes.models import ContentType
from django.views.generic import TemplateView


class ContentIndexView(TemplateView):

    """Displays the index page for a page."""

    def get_template_names(self):
        """Returns the list of template names."""
        content_cls = ContentType.objects.get_for_id(self.request.pages.current.content_type_id).model_class()
        params = {
            "model_name": content_cls.__name__.lower(),
            "app_label": content_cls._meta.app_label,
        }
        return (
            "{app_label}/{model_name}.html".format(**params),
            "{app_label}/base.html".format(**params),
            "base.html",
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0003_auto_20150508_1504'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='country',
            options={'ordering': ('name',), 'verbose_name_plural': 'countries'},
        ),
        migrations.AlterModelOptions(
            name='countrygroup',
            options={'ordering': ('name',)},
        ),
        migrations.RemoveField(
            model_name='countrygroup',
            name='default',
        ),
    ]



"""
Production settings for {{ project_name }} project.

For an explanation of these settings, please see the Django documentation at:

<http://docs.djangoproject.com/en/dev/>

While many of these settings assume sensible defaults, you must provide values
for the site, database, media and email sections below.
"""
from __future__ import unicode_literals

from social.pipeline import DEFAULT_AUTH_PIPELINE

import os
import sys


# The name of this site.  Used for branding in the online admin area.

SITE_NAME = "Example"

SITE_DOMAIN = "example.com"

PREPEND_WWW = True

ALLOWED_HOSTS = [
    SITE_DOMAIN,
    'www.{}'.format(SITE_DOMAIN)
]

SUIT_CONFIG = {
    'ADMIN_NAME': SITE_NAME
}

# Database settings.

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql_psycopg2",
        "NAME": "{{ project_name }}",
        "USER": "{{ project_name }}",
        "PASSWORD": "",
        "HOST": "",
        "PORT": ""
    }
}


# Absolute path to the directory where all uploaded media files are stored.

MEDIA_ROOT = "/var/www/{{ project_name }}_media"

MEDIA_URL = "/media/"

FILE_UPLOAD_PERMISSIONS = 0o644


# Absolute path to the directory where static files will be collected.

STATIC_ROOT = "/var/www/{{ project_name }}_static"

STATIC_URL = "/static/"


# Email settings.

EMAIL_HOST = "smtp.mandrillapp.com"

EMAIL_HOST_USER = "developers@onespacemedia.com"

EMAIL_HOST_PASSWORD = ""

EMAIL_PORT = 587

EMAIL_USE_TLS = True

SERVER_EMAIL = "{name} <notifications@{domain}>".format(
    name=SITE_NAME,
    domain=SITE_DOMAIN,
)

DEFAULT_FROM_EMAIL = SERVER_EMAIL

EMAIL_SUBJECT_PREFIX = "[%s] " % SITE_NAME


# Error reporting settings.  Use these to set up automatic error notifications.

ADMINS = (
    ("Onespacemedia Errors", "errors@onespacemedia.com"),
)

MANAGERS = ()

SEND_BROKEN_LINK_EMAILS = False


# Locale settings.

TIME_ZONE = "Europe/London"

LANGUAGE_CODE = "en-gb"

USE_I18N = False

USE_L10N = True

USE_TZ = True


# Auto-discovery of project location.

SITE_ROOT = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))


# A list of additional installed applications.

INSTALLED_APPS = (

    "django.contrib.sessions",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "suit",
    "django.contrib.admin",
    "django.contrib.sitemaps",

    "sorl.thumbnail",
    "compressor",

    "cms",

    "reversion",
    "usertools",
    # "historylinks",
    "watson",

    "cms.apps.pages",
    "cms.apps.links",
    "cms.apps.media",
    "cms.apps.news",

    "{{ project_name }}.apps.site",
    "{{ project_name }}.apps.faqs",
    "{{ project_name }}.apps.jobs",
    "{{ project_name }}.apps.people",

    'server_management',
    'django_extensions',
    'cachalot',

    'social.apps.django_app.default',
)


# Additional static file locations.

STATICFILES_DIRS = (
    os.path.join(SITE_ROOT, "static"),
)

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'compressor.finders.CompressorFinder',
)

COMPRESS_CSS_FILTERS = [
    'compressor.filters.css_default.CssAbsoluteFilter',
    'compressor.filters.cssmin.CSSMinFilter'
]

# Dispatch settings.

MIDDLEWARE_CLASSES = (
    "django.middleware.common.CommonMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "watson.middleware.SearchContextMiddleware",
    # "historylinks.middleware.HistoryLinkFallbackMiddleware",
    "cms.middleware.PublicationMiddleware",
    "cms.apps.pages.middleware.PageMiddleware",
)

PASSWORD_HASHERS = (
    'django.contrib.auth.hashers.BCryptSHA256PasswordHasher',
    'django.contrib.auth.hashers.BCryptPasswordHasher',
    'django.contrib.auth.hashers.PBKDF2PasswordHasher',
    'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
    'django.contrib.auth.hashers.SHA1PasswordHasher',
    'django.contrib.auth.hashers.MD5PasswordHasher',
    'django.contrib.auth.hashers.CryptPasswordHasher',
)


ROOT_URLCONF = "{{ project_name }}.urls"

WSGI_APPLICATION = "{{ project_name }}.wsgi.application"

PUBLICATION_MIDDLEWARE_EXCLUDE_URLS = (
    "^admin/.*",
)

SESSION_ENGINE = "django.contrib.sessions.backends.signed_cookies"

MESSAGE_STORAGE = "django.contrib.messages.storage.cookie.CookieStorage"

SITE_ID = 1

# Absolute path to the directory where templates are stored.

TEMPLATE_DIRS = (
    os.path.join(SITE_ROOT, "templates"),
)

TEMPLATE_LOADERS = (
    ("django.template.loaders.cached.Loader", (
        "django.template.loaders.filesystem.Loader",
        "django.template.loaders.app_directories.Loader",
    )),
)

TEMPLATE_CONTEXT_PROCESSORS = (
    "django.contrib.auth.context_processors.auth",
    "django.core.context_processors.debug",
    "django.core.context_processors.i18n",
    "django.core.context_processors.media",
    "django.core.context_processors.static",
    "django.contrib.messages.context_processors.messages",
    "django.core.context_processors.request",
    "cms.context_processors.settings",
    "cms.apps.pages.context_processors.pages",
    'social.apps.django_app.context_processors.backends',
    'social.apps.django_app.context_processors.login_redirect',
)


# Namespace for cache keys, if using a process-shared cache.

CACHE_MIDDLEWARE_KEY_PREFIX = "{{ project_name }}"

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.memcached.MemcachedCache",
        'LOCATION': '127.0.0.1:11211',
    }
}


# A secret key used for cryptographic algorithms.

SECRET_KEY = "{{ secret_key }}"


REDACTOR_OPTIONS = {
    "plugins": ["table", "imagemanager", "video", "filemanager"],
    "imageUpload": "/admin/media/file/redactor/upload/image/",
    "fileUpload": "/admin/media/file/redactor/upload/file/",
    "minHeight": 300,
    "formattingAdd": [
        {
            "tag": "a",
            "title": "Button",
            "class": "button primary",
        }
    ]
}


NEWS_APPROVAL_SYSTEM = False

GOOGLE_ANALYTICS = ''

# You can get your Client ID & Secret here: https://creativesdk.adobe.com/myapps.html
ADOBE_CREATIVE_SDK_ENABLED = False
ADOBE_CREATIVE_SDK_CLIENT_SECRET = ''
ADOBE_CREATIVE_SDK_CLIENT_ID = ''

# Google Apps authentication.

# SETUP:
# 1. https://console.developers.google.com/project
# 2. "Create project"
# 3. APIs & auth -> Consent screen
# 4. Select email address
# 5. APIs & auth -> APIs
# 6. Enable "Google+ API"
# 7. APIs & auth -> Credentials
# 8. Create new Client ID -> Web application
# 9. Copy Client ID to KEY below.
# 10. Copy Client Secret to SECRET below.
# 11. Edit settings
# 12. Set authorized domain

AUTHENTICATION_BACKENDS = (
    'social.backends.google.GooglePlusAuth',
    'django.contrib.auth.backends.ModelBackend'
)

SOCIAL_AUTH_GOOGLE_PLUS_KEY = ''
SOCIAL_AUTH_GOOGLE_PLUS_SECRET = ''
WHITELISTED_DOMAINS = ['onespacemedia.com']
SOCIAL_AUTH_PROTECTED_USER_FIELDS = ['first_name', 'last_name']

SOCIAL_AUTH_LOGIN_REDIRECT_URL = '/admin/'
SOCIAL_AUTH_PIPELINE = DEFAULT_AUTH_PIPELINE + (
    'cms.pipeline.make_staff',
)

SILENCED_SYSTEM_CHECKS = [
    '1_6.W001',
    # '1_6.W002'
]

if 'test' in sys.argv:
    # The CMS tests use test-only models, which won't be loaded if we only load
    # our real migration files, so point to a nonexistent one, which will make
    # the test runner fall back to 'syncdb' behavior.

    # Note: This will not catch a situation where a developer commits model
    # changes without the migration files.

    class DisableMigrations(object):

        def __contains__(self, item):
            return True

        def __getitem__(self, item):
            return "notmigrations"

    MIGRATION_MODULES = DisableMigrations()


GEOIP_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), "../geoip/"))



from cms.models import HtmlField
from cms.apps.pages.models import ContentBase


# class Content(ContentBase):

#     content_primary = HtmlField(
#         "primary content",
#         blank=True
#     )



"""Template tags used to render pages."""
from __future__ import unicode_literals

from django import template
from django.utils.html import escape

from cms.apps.pages.models import Page


register = template.Library()


# Navigation.

@register.inclusion_tag("pages/navigation.html", takes_context=True)
def navigation(context, pages, section=None):
    """
    Renders a navigation list for the given pages.

    The pages should all be a subclass of PageBase, and possess a get_absolute_url() method.

    You can also specify an alias for the navigation, at which point it will be set in the
    context rather than rendered.
    """
    request = context["request"]
    # Compile the entries.

    def page_entry(page):
        url = page.get_absolute_url()
        return {
            "url": url,
            "page": page,
            "title": str(page),
            "here": request.path.startswith(url),
        }
    entries = map(page_entry, pages)
    # Add the section.
    if section:
        section_entry = page_entry(section)
        section_entry["here"] = context["pages"].current == section_entry["page"]
        entries = [section_entry] + list(entries)
    # Render the template.
    context.update({
        "request": request,
        "navigation": entries,
    })
    return context


@register.assignment_tag(takes_context=True)
def get_navigation(context, pages, section=None):
    """Returns a navigation list for the given pages."""
    return navigation(context, pages, section)["navigation"]


# Page linking.

@register.simple_tag
def page_url(page, view_func=None, *args, **kwargs):
    """Renders the URL of the given view func in the given page."""
    url = None
    if isinstance(page, int):
        try:
            page = Page.objects.get(pk=page)
        except Page.DoesNotExist:
            url = "#"
            page = None
    if page is None:
        url = "#"
    else:
        # Get the page URL.
        if view_func is None:
            url = page.get_absolute_url()
        else:
            url = page.reverse(view_func, args, kwargs)
    # Return the value, or set as a context variable as appropriate.
    return escape(url)


# Page widgets.

@register.simple_tag(takes_context=True)
def meta_description(context, description=None):
    """
    Renders the content of the meta description tag for the current page::

        {% meta_description %}

    You can override the meta description by setting a context variable called
    'meta_description'::

        {% with "foo" as meta_description %}
            {% meta_description %}
        {% endwith %}

    You can also provide the meta description as an argument to this tag::

        {% meta_description "foo" %}

    """
    if description is None:
        description = context.get("meta_description")
    if description is None:
        request = context["request"]
        page = request.pages.current
        if page:
            description = page.meta_description
    return escape(description or "")


@register.simple_tag(takes_context=True)
def meta_keywords(context, keywords=None):
    """
    Renders the content of the meta keywords tag for the current page::

        {% meta_keywords %}

    You can override the meta keywords by setting a context variable called
    'meta_keywords'::

        {% with "foo" as meta_keywords %}
            {% meta_keywords %}
        {% endwith %}

    You can also provide the meta keywords as an argument to this tag::

        {% meta_keywords "foo" %}

    """
    if keywords is None:
        keywords = context.get("meta_keywords")
    if keywords is None:
        request = context["request"]
        page = request.pages.current
        if page:
            keywords = page.meta_keywords
    return escape(keywords or "")


@register.simple_tag(takes_context=True)
def meta_robots(context, index=None, follow=None, archive=None):
    """
    Renders the content of the meta robots tag for the current page::

        {% meta_robots %}

    You can override the meta robots by setting boolean context variables called
    'robots_index', 'robots_archive' and 'robots_follow'::

        {% with 1 as robots_follow %}
            {% meta_robots %}
        {% endwith %}

    You can also provide the meta robots as three boolean arguments to this
    tag in the order 'index', 'follow' and 'archive'::

        {% meta_robots 1 1 1 %}

    """
    # Override with context variables.
    if index is None:
        index = context.get("robots_index")
    if follow is None:
        follow = context.get("robots_follow")
    if archive is None:
        archive = context.get("robots_archive")
    # Final override, set to True.
    if index is None:
        index = True
    if follow is None:
        follow = True
    if archive is None:
        archive = True
    # Generate the meta content.
    robots = ", ".join((
        index and "INDEX" or "NOINDEX",
        follow and "FOLLOW" or "NOFOLLOW",
        archive and "ARCHIVE" or "NOARCHIVE",
    ))
    return escape(robots)


@register.inclusion_tag("pages/title.html", takes_context=True)
def title(context, browser_title=None):
    """
    Renders the title of the current page::

        {% title %}

    You can override the title by setting a context variable called 'title'::

        {% with "foo" as title %}
            {% title %}
        {% endwith %}

    You can also provide the title as an argument to this tag::

        {% title "foo" %}

    """
    request = context["request"]
    page = request.pages.current
    homepage = request.pages.homepage
    # Render the title template.
    return {
        "title": browser_title or context.get("title") or (page and page.browser_title) or (page and page.title) or "",
        "site_title": (homepage and homepage.browser_title) or (homepage and homepage.title) or ""
    }


@register.inclusion_tag("pages/breadcrumbs.html", takes_context=True)
def breadcrumbs(context, page=None, extended=False):
    """
    Renders the breadcrumbs trail for the current page::

        {% breadcrumbs %}

    To override and extend the breadcrumb trail within page applications, add
    the 'extended' flag to the tag and add your own breadcrumbs underneath::

        {% breadcrumbs extended=1 %}

    """
    request = context["request"]
    # Render the tag.
    page = page or request.pages.current
    if page:
        breadcrumb_list = [{
            "short_title": breadcrumb.short_title or breadcrumb.title,
            "title": breadcrumb.title,
            "url": breadcrumb.get_absolute_url(),
            "last": False,
            "page": breadcrumb,
        } for breadcrumb in request.pages.breadcrumbs]
    else:
        breadcrumb_list = []
    if not extended:
        breadcrumb_list[-1]["last"] = True
    # Render the breadcrumbs.
    return {
        "breadcrumbs": breadcrumb_list,
    }


@register.inclusion_tag("pages/header.html", takes_context=True)
def header(context, page_header=None):
    """
    Renders the header for the current page::

        {% header %}

    You can override the page header by providing a 'header' or 'title' context
    variable. If both are present, then 'header' overrides 'title'::

        {% with "foo" as header %}
            {% header %}
        {% endwith %}

    You can also provide the header as an argument to this tag::

        {% header "foo" %}

    """
    request = context["request"]
    page_header = page_header or context.get("header") or context.get("title") or request.pages.current.title
    return {
        "header": page_header,
    }



"""Core models used by the CMS."""


from cms.models.base import PageBase, PublishedBase, PublishedBaseSearchAdapter, SearchMetaBase, OnlineBase, OnlineBaseSearchAdapter, SearchMetaBaseSearchAdapter, PageBaseSearchAdapter
from cms.models.fields import HtmlField, LinkField
from cms.models.managers import PublicationManagementError, publication_manager,  PublishedBaseManager, OnlineBaseManager, SearchMetaBaseManager, PageBaseManager



from django.contrib.contenttypes.models import ContentType
from django.test import TestCase, RequestFactory

from ..middleware import RequestPageManager
from ..models import ContentBase, Page
from ..templatetags.pages import get_navigation, page_url, meta_keywords, breadcrumbs
from .... import externals


class TestTemplatetagPage(ContentBase):
    urlconf = 'cms.apps.pages.tests.urls'


class TestTemplatetags(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.request = self.factory.get('/')

        with externals.watson.context_manager("update_index")():
            content_type = ContentType.objects.get_for_model(TestTemplatetagPage)

            self.homepage = Page.objects.create(
                title="Homepage",
                url_title='homepage',
                content_type=content_type,
            )

            TestTemplatetagPage.objects.create(
                page=self.homepage,
            )

            self.section = Page.objects.create(
                parent=self.homepage,
                title="Section",
                url_title='section',
                content_type=content_type,
            )

            TestTemplatetagPage.objects.create(
                page=self.section,
            )

            self.subsection = Page.objects.create(
                parent=self.section,
                title="Subsection",
                content_type=content_type,
            )

            TestTemplatetagPage.objects.create(
                page=self.subsection,
            )

            self.subsubsection = Page.objects.create(
                parent=self.subsection,
                title="Subsubsection",
                content_type=content_type,
            )

            TestTemplatetagPage.objects.create(
                page=self.subsubsection,
            )

    def test_get_navigation(self):
        self.homepage = Page.objects.get(pk=self.homepage.pk)
        navigation = get_navigation({'request': self.request}, self.homepage.navigation)

        self.assertListEqual(list(navigation), [
            {
                'url': '/section/',
                'page': self.section,
                'here': False,
                'title': 'Section'
            }
        ])

    def test_page_url(self):
        self.assertEqual(page_url(self.homepage), '/')
        self.assertEqual(page_url(self.homepage.pk), '/')
        self.assertEqual(page_url(-1), '#')
        self.assertEqual(page_url(None), '#')
        self.assertEqual(
            page_url(self.homepage.pk, 'detail', url_title='homepage'),
            '/homepage/'
        )

    def test_meta_keywords(self):
        self.request.pages = RequestPageManager(self.factory.get(''))

        self.assertEqual(meta_keywords({'request': self.request}), '')
        self.assertEqual(meta_keywords({
            'request': self.request,
            'meta_keywords': 'Foo'
        }), 'Foo')

    def test_breadcrumbs(self):
        class Object(object):
            current = None

        self.request.pages = Object()

        output = breadcrumbs({'request': self.request}, extended=True)
        self.assertDictEqual(output, {
            'breadcrumbs': []
        })



from django.contrib.contenttypes.models import ContentType
from django.test import TestCase
from django.template import VariableDoesNotExist
from django.utils.timezone import now

from ..models import Article, Category, NewsFeed
from ..templatetags.news import (page_context, get_page_from_context,
                                 takes_current_page, takes_article_page,
                                 article_list, article_url, article_list_item,
                                 article_archive_url, category_url,
                                 category_list, article_year_archive_url,
                                 article_day_archive_url, article_date,
                                 article_category_list, article_meta,
                                 article_date_list, article_latest_list,
                                 get_article_latest_list)
from ...pages.models import ContentBase, Page
from .... import externals


class TestPageContent(ContentBase):
    pass


class Object(object):
    pass


class NewsTest(TestCase):

    def setUp(self):
        with externals.watson.context_manager("update_index")():
            content_type = ContentType.objects.get_for_model(TestPageContent)

            self.homepage = Page.objects.create(
                title="Homepage",
                url_title='homepage',
                content_type=content_type,
            )

            TestPageContent.objects.create(
                page=self.homepage,
            )

    def _create_feed_article(self):
        self.date = now()
        self.date_str = '/{}/{}/{}'.format(
            self.date.strftime('%Y'),
            self.date.strftime('%b').lower(),
            self.date.strftime('%d').lstrip('0'),
        )

        # Create a NewsFeed page.
        with externals.watson.context_manager("update_index")():
            content_type = ContentType.objects.get_for_model(NewsFeed)

            self.page = Page.objects.create(
                title="News Feed",
                url_title='news',
                parent=self.homepage,
                content_type=content_type,
            )

            self.feed = NewsFeed.objects.create(
                page=self.page,
            )

            self.category = Category.objects.create(
                url_title='foo'
            )

        # Create an Article.
        self.article = Article.objects.create(
            news_feed=self.feed,
            title='Article',
            url_title='foo',
            date=self.date,
        )

    def test_page_context(self):
        def inner_function(context):
            return context

        initial_data = {
            'request': {},
            'pages': {},
            'page': {}
        }

        context = page_context(inner_function)(initial_data)

        self.assertDictEqual(context, initial_data)

    def test_get_page_from_context(self):
        # This might seem odd, but remember that we are in the News app, we're
        # trying to get the NewsFeed, not just the current page.

        context = {}
        kwargs = {}

        self.assertIsNone(get_page_from_context(context, kwargs))

        kwargs['page'] = self.homepage.pk

        self.assertIsNone(get_page_from_context(context, kwargs))

        del kwargs['page']
        context['page'] = self.homepage.pk
        self.assertIsNone(get_page_from_context(context, kwargs))

        del context['page']
        context['pages'] = Object()
        context['pages'].current = self.homepage.pk
        self.assertIsNone(get_page_from_context(context, kwargs))

    def test_takes_current_page(self):
        def inner_function(context, page):
            return page

        with self.assertRaises(VariableDoesNotExist):
            takes_current_page(inner_function)({})

        # Create a NewsFeed page.
        with externals.watson.context_manager("update_index")():
            content_type = ContentType.objects.get_for_model(NewsFeed)

            self.page = Page.objects.create(
                title="News Feed",
                parent=self.homepage,
                content_type=content_type,
            )

            self.feed = NewsFeed.objects.create(
                page=self.page,
            )

        self.assertEqual(takes_current_page(inner_function)({}), self.page)

    def test_takes_article_page(self):
        def inner_function(context, article, page):
            self.assertEqual(context, {})
            self.assertEqual(article, self.article)
            self.assertEqual(page, self.page)
            return article

        self._create_feed_article()

        takes_article_page(inner_function)({}, self.article)

    def test_article_list(self):
        response = article_list({'request': {}}, {})

        self.assertDictEqual(response, {
            'article_list': {},
            'page_obj': None,
            'request': {}
        })

    def test_article_url(self):
        self._create_feed_article()

        url = article_url({}, self.article, page=self.page)
        self.assertEqual(url, '/news' + self.date_str + '/foo/')

    def test_article_list_item(self):
        self._create_feed_article()

        item = article_list_item({'request': {}}, self.article, page=self.page)

        self.assertDictEqual(item, {
            'article': self.article,
            'request': {},
            'page': self.page
        })

    def test_article_archive_url(self):
        self._create_feed_article()
        archive_url = article_archive_url({'request': {}}, page=self.page)
        self.assertEqual(archive_url, '/news/')

    def test_category_url(self):
        self._create_feed_article()
        self.assertEqual(category_url({}, self.category, page=self.page), '/news/foo/')

    def test_category_list(self):
        cat_list = category_list({'request': {}}, list(Category.objects.all()))
        self.assertDictEqual(cat_list, {
            'category': None,
            'current_category': None,
            'request': {},
            'category_list': []
        })

        self._create_feed_article()

        cat_list = category_list({'request': {}}, list(Category.objects.all()))
        self.assertDictEqual(cat_list, {
            'category': None,
            'current_category': None,
            'request': {},
            'category_list': [self.category]
        })

    def test_article_year_archive_url(self):
        self._create_feed_article()
        url = article_year_archive_url({}, self.date.year, page=self.page)
        self.assertEqual(url, '/news/' + str(self.date.year) + '/')

    def test_article_day_archive_url(self):
        self._create_feed_article()
        url = article_day_archive_url({}, self.date, page=self.page)
        self.assertEqual(url, '/news' + self.date_str + '/')

    def test_article_date(self):
        self._create_feed_article()
        date = article_date({'request': {}}, self.article)

        self.assertDictEqual(date, {
            'article': self.article,
            'request': {}
        })

    def test_article_category_list(self):
        self._create_feed_article()
        category_list = article_category_list({'request': {}}, self.article)

        self.assertEqual(category_list['article'], self.article)
        self.assertEqual(category_list['request'], {})
        self.assertEqual(len(category_list['categories']), 0)

    def test_article_meta(self):
        self._create_feed_article()
        meta = article_meta({'request': {}}, self.article)

        self.assertDictEqual(meta, {
            'article': self.article,
            'request': {}
        })

    def test_article_date_list(self):
        self._create_feed_article()
        date_list = article_date_list({'request': {}}, page=self.page)

        self.assertEqual(date_list['request'], {})
        self.assertIsNone(date_list['current_year'])
        self.assertEqual(list(date_list['date_list']), [self.date.date().replace(day=1)])

        date_list = article_date_list({'request': {}, 'year': 1970}, page=self.page)

        self.assertEqual(date_list['request'], {})
        self.assertEqual(date_list['current_year'], 1970)
        self.assertEqual(list(date_list['date_list']), [self.date.date().replace(day=1)])

        date_list = article_date_list({'request': {}, 'year': self.date.date()}, page=self.page)

        self.assertEqual(date_list['request'], {})
        self.assertEqual(date_list['current_year'], self.date.date().year)
        self.assertEqual(list(date_list['date_list']), [self.date.date().replace(day=1)])

        date_list = article_date_list({'request': {}, 'month': self.date.date()}, page=self.page)

        self.assertEqual(date_list['request'], {})
        self.assertEqual(date_list['current_year'], self.date.date().year)
        self.assertEqual(list(date_list['date_list']), [self.date.date().replace(day=1)])

        date_list = article_date_list({'request': {}, 'day': self.date.date()}, page=self.page)

        self.assertEqual(date_list['request'], {})
        self.assertEqual(date_list['current_year'], self.date.date().year)
        self.assertEqual(list(date_list['date_list']), [self.date.date().replace(day=1)])

        date_list = article_date_list({'request': {}, 'article': self.article}, page=self.page)

        self.assertEqual(date_list['request'], {})
        self.assertEqual(date_list['current_year'], self.date.date().year)
        self.assertEqual(list(date_list['date_list']), [self.date.date().replace(day=1)])

    def test_article_latest_list(self):
        self._create_feed_article()
        latest_list = article_latest_list({'request': {}}, page=self.page)

        self.assertListEqual(list(latest_list['article_list']), [self.article])
        self.assertDictEqual(latest_list['request'], {})
        self.assertEqual(latest_list['page'], self.page)

    def test_get_article_latest_list(self):
        self._create_feed_article()
        latest_list = get_article_latest_list({'request': {}}, page=self.page)
        self.assertListEqual(list(latest_list), [self.article])
